# clm version and release
%define clmver  3
%define pkgver  2008.02.25
%define pkgrel  4%{?dist}
%define ver     %{nil}

# which lisp we are building for
%ifarch x86_64
%define cmucl_build 0
%else
%if 0%{?fedora} < 9
%define cmucl_build 1
%else
%define cmucl_build 0
%endif
%endif
%define sbcl_build  1

# lisp versions and lisp specific clm package releases
%if 0%{?fedora} == 11
%define cmucl_ver 19f
%define sbcl_ver  1.0.30
%define sbcl_rel  1
%endif
%if 0%{?fedora} == 10
%define cmucl_ver 19e
%define sbcl_ver  1.0.30
%define sbcl_rel  1
%endif
%if 0%{?fedora} == 9
%define cmucl_ver 19e
%define sbcl_ver  1.0.22
%define sbcl_rel  1
%endif
%if 0%{?rhel} == 5
%define cmucl_ver  19d
%define sbcl_ver   1.0.30
%define sbcl_rel   1
%endif
%if 0%{?rhel} == 4
%define cmucl_ver  19d
%define sbcl_ver   1.0.5
%define sbcl_rel   1
%endif

# --without alsa: build for oss api (default is alsa)
%define alsa        %{?_without_alsa:0}%{!?_without_alsa:1}

# --without-jack: disable jack support
%define jack        %{?_without_jack:0}%{!?_without_jack:1}

# various locations
%define cmucl_clm_bin_dir %{_libdir}/cmucl/lib/clm%{ver}
%define cmucl_subsystems  %{_libdir}/cmucl/lib/subsystems
%define sbcl_clm_bin_dir  %{_libdir}/sbcl/clm%{ver}
%define sbcl_subsystems   %{_libdir}/sbcl/site-systems
%define clm_src_dir       %{_datadir}/common-lisp/source/clm%{ver}

Summary: Common Lisp Music (CLM-%{clmver}, %{version})
Name: clm
Version: %{pkgver}
Release: %{pkgrel}
License: GPL
Group: Development/Languages
URL: http://ccrma.stanford.edu/software/clm
Source0: clm-%{clmver}.tar.gz
Source1: asdf.lisp
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
Packager: Fernando Lopez-Lezcano
Vendor: Planet CCRMA
Distribution: Planet CCRMA

# needed by the build scripts
Requires: tcsh

Obsoletes: clm-source

%if "%{cmucl_build}" == "1"
BuildRequires: cmucl = %{cmucl_ver}
%endif
%if "%{sbcl_build}" == "1"
BuildRequires: sbcl = %{sbcl_ver}, tcsh
%endif
%if 0%{?fedora} >= 5 || 0%{?rhel} >= 5
BuildRequires: xorg-x11-proto-devel
%else
BuildRequires: XFree86-devel
%endif
%if 0%{?fedora} >= 11
BuildRequires: libX11-devel
%endif
BuildRequires: alsa-lib-devel libsamplerate-devel
BuildRequires: jack-audio-connection-kit-devel

%description
This package contains CLM-%{clmver} [%{version}] (Common Lisp Music)
source and documentation.

Common Lisp Music, by Bill Schottstaedt (CLM, for lack of a catchier
name), is a music synthesis and signal processing package in the Music
V family.

%package   cmucl
Summary:   Common Lisp Music binaries for CMUCL (CLM-%{clmver}, %{version})
Group:     Development/Languages
Requires:  cmucl = %{cmucl_ver}
Requires:  clm%{ver} = %{version}-%{release}
Requires:  gcc
Obsoletes: cmucl-clm cmucl-cm-clm cmucl-cm-clm-cmn

%description cmucl
This package contains CLM-%{clmver} [%{version}] (Common Lisp Music)
binaries built on top of CMUCL Common Lisp.

%package   sbcl
Summary:   Common Lisp Music binaries for SBCL (CLM-%{clmver}, %{version})
Group:     Development/Languages
Requires:  sbcl >= %{sbcl_ver}-%{sbcl_rel}
Requires:  clm%{ver} = %{version}-%{release}
Requires:  gcc

%description sbcl
This package contains CLM-%{clmver} [%{version}] (Common Lisp Music) 
built on top of SBCL Common Lisp.

%prep
%setup -q -n clm-%{clmver}

%build
# add the --build-id flag to the linker
%{__perl} -p -i -e 's|shared-name " "|shared-name " --build-id "|g' all.lisp

# save a copy of the patched source
%{__tar} cf .clm.tar *

# SBCL
%if %{sbcl_build}
%{__mkdir} bin-sbcl
# define a short lisp program to load clm
# source and bin paths and compiler message muffling is patched into all.lisp later
%{__cat} <<EOF >clm%{ver}-sbcl-load.lisp
;;;
;;; SBCL Common Lisp Music (CLM%{ver}) startup code
;;;
(format t ";;;~%;;; Loading CLM-%{clmver} %{version}...~%;;;~%")
(require :asdf)
(push "%{clm_src_dir}/" asdf:*central-registry*)
(asdf:operate 'asdf:load-source-op :clm)
(setf clm::*clm-player* "%{sbcl_clm_bin_dir}/sndplay")
(format t ";;;~%;;; CLM-%{clmver} %{version} loaded~%;;;~%")
EOF

# build clm
/usr/bin/sbcl <<EOF
;; turn off some error reporting, see 
;; http://sbcl.sourceforge.net/manual/Controlling-Verbosity.html
(setf *compile-print* nil)
(declaim (sb-ext:muffle-conditions style-warning sb-ext:compiler-note))
(declaim (sb-ext:muffle-conditions sb-ext:code-deletion-note))
(in-package :cl-user)
%if "%{alsa}" == "1"
(push :alsa *features*)
%endif
%if "%{jack}" == "1"
(push :jack *features*)
%endif
(defparameter clm-directory "./")
(defparameter clm-bin-directory "bin-sbcl/")
(require :asdf)
(asdf:operate 'asdf:load-source-op :clm)
(compile-file "clm%{ver}-sbcl-load.lisp")
(quit)
EOF
%endif

# CMUCL
%if %{cmucl_build}
%{__mkdir} bin-cmucl
# install the asdf source
%{__cp} %{SOURCE1} .
# define the cmucl clm loader
%{__cat} <<EOF >clm%{ver}-cmucl-load.lisp
;;;
;;; CMUCL Common Lisp Music (CLM%{ver}) startup code
;;;
(format t ";;;~%;;; Loading CLM-%{clmver} %{version}...~%;;;~%")
;;; load clm with asdf
(require :asdf (make-pathname :name "asdf" :directory (pathname-directory "%{cmucl_clm_bin_dir}/")))
(push "%{clm_src_dir}/" asdf:*central-registry*)
(asdf:operate 'asdf:load-source-op :clm)
(setf clm::*clm-player* "%{cmucl_clm_bin_dir}/sndplay")
(format t ";;;~%;;; CLM-%{clmver} %{version} loaded~%;;;~%")
EOF

# build clm
/usr/bin/lisp <<EOF
;; fix problems with signed arrays
(pushnew :signed-array *features*)
(setf *compile-print* nil)
(in-package :cl-user)
%if "%{alsa}" == "1"
(push :alsa *features*)
%endif
%if "%{jack}" == "1"
(push :jack *features*)
%endif
(defparameter clm-directory "./")
(defparameter clm-bin-directory "bin-cmucl/")
;;; we use asdf.lisp from cm
(compile-file "asdf.lisp")
(require :asdf (make-pathname :name "asdf"))
(asdf:operate 'asdf:load-source-op :clm)
(quit)
EOF
%endif

%install
%{__rm} -rf %{buildroot}

%{__mkdir} -p %{buildroot}%{_bindir}
%{__mkdir} -p %{buildroot}%{clm_src_dir}
%{__mkdir} -p %{buildroot}%{_docdir}

# copy the patched source tree, this is going to be *clm-directory*
(cd %{buildroot}%{clm_src_dir}; \
 %{__tar} xpf %{_builddir}/clm-%{clmver}/.clm.tar;)

# SBCL install
%if %{sbcl_build}
%{__mkdir} -p %{buildroot}%{sbcl_clm_bin_dir}

(cd %{_builddir}/clm-%{clmver}/bin-sbcl; %{__tar} cf - *)| \
(cd %{buildroot}%{sbcl_clm_bin_dir}; %{__tar} xpf -)

# install the configuration stuff in the source directory
%{__cp} -p configure mus-config.h sndlib.h %{buildroot}%{clm_src_dir}

# patch all.lisp with proper startup paths, this is needed so that
# clm can be loaded with (use-system ...) from within common music,
# also add compiler message muffling
%{__perl} -p -i -e "s|\(if \(not \(boundp 'clm-directory\)\)|\#\+\(or sbcl cmu\) \(setf \*compile-print\* nil\)\n\#\+sbcl \(declaim \(sb-ext\:muffle-conditions style-warning sb-ext:compiler-note\)\)\n\\#\+sbcl (declaim \(sb-ext\:muffle-conditions sb-ext\:code-deletion-note\)\)\n\n\(defparameter clm-directory \"%{clm_src_dir}/\"\)\n\(defparameter clm-bin-directory \#\+sbcl \"%{sbcl_clm_bin_dir}/\" \#\+cmu \"%{cmucl_clm_bin_dir}/\"\) \n\n\(if \(not \(boundp 'clm-directory\)\)|" %{buildroot}%{clm_src_dir}/all.lisp

# install the startup program
%{__cp} -p clm%{ver}-sbcl-load.lisp %{buildroot}%{clm_src_dir}/clm%{ver}.lisp
%{__cp} -p clm%{ver}-sbcl-load.fasl %{buildroot}%{sbcl_clm_bin_dir}/clm%{ver}.fasl

# the clm startup script
%{__cat} <<EOF >%{buildroot}%{_bindir}/clm%{ver}-sbcl
#!/bin/bash
# CLM%{ver} startup script for sbcl
exec /usr/bin/sbcl --load "%{sbcl_clm_bin_dir}/clm%{ver}" $@
EOF
%{__chmod} 0755 %{buildroot}%{_bindir}/clm%{ver}-sbcl
%endif

# CMUCL install
%if %{cmucl_build}
%{__mkdir} -p %{buildroot}%{cmucl_clm_bin_dir}

(cd %{_builddir}/clm-%{clmver}/bin-cmucl; %{__tar} cf - *)| \
(cd %{buildroot}%{cmucl_clm_bin_dir}; %{__tar} xpf -)

# install asdf
%{__cp} -p asdf.lisp %{buildroot}%{clm_src_dir}/
%{__cp} -p asdf.x86f %{buildroot}%{cmucl_clm_bin_dir}/

# install the configuration stuff in the source directory
%{__cp} -p configure mus-config.h sndlib.h %{buildroot}%{clm_src_dir}

# patch all.lisp with proper startup paths, this is needed so that
# clm can be loaded with (use-system ...) within common music
%{__perl} -p -i -e "s|\(if \(not \(boundp 'clm-directory\)\)|\#\+\(or sbcl cmu\) \(setf \*compile-print\* nil\)\n\#\+sbcl \(declaim \(sb-ext\:muffle-conditions style-warning sb-ext:compiler-note\)\)\n\\#\+sbcl (declaim \(sb-ext\:muffle-conditions sb-ext\:code-deletion-note\)\)\n\n\(defparameter clm-directory \"%{clm_src_dir}/\"\)\n\(defparameter clm-bin-directory \#\+sbcl \"%{sbcl_clm_bin_dir}/\" \#\+cmu \"%{cmucl_clm_bin_dir}/\"\) \n\n\(if \(not \(boundp 'clm-directory\)\)|" %{buildroot}%{clm_src_dir}/all.lisp

# install the startup program, for some reason in cmucl I can't make
# a compiled version of this load correctly (some problem with asdf) 
# so for now use the plain .lisp file
%{__cp} -p clm%{ver}-cmucl-load.lisp %{buildroot}%{cmucl_clm_bin_dir}/clm%{ver}.lisp

# the clm startup script
%{__cat} <<EOF >%{buildroot}%{_bindir}/clm%{ver}-cmucl
#!/bin/bash
# CLM%{ver} startup script for cmucl
exec /usr/bin/lisp -load "%{cmucl_clm_bin_dir}/clm%{ver}.lisp" $@
EOF
%{__chmod} 0755 %{buildroot}%{_bindir}/clm%{ver}-cmucl
%endif

%clean
%{__rm} -rf %{buildroot}

%files
%defattr(-,root,root,-)
%doc
%{clm_src_dir}

%if %{cmucl_build}
%files cmucl
%defattr(-,root,root,-)
%{cmucl_clm_bin_dir}
%{_bindir}/*cmucl
%endif

%if %{sbcl_build}
%files sbcl
%defattr(-,root,root,-)
%{sbcl_clm_bin_dir}
%{_bindir}/*sbcl
%endif

%changelog
* Tue Sep 29 2009 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-4
- updated lisp versions for CentOS 5

* Wed Sep  2 2009 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-4
- updated lisp versions for Fedora 10/11

* Wed Jun 10 2009 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-3
- updated lisp versions for Fedora 11
- add libX11-devel build requirement, fixed bad code that triggered
  a failed build

* Mon Dec  3 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-3
- updated sbcl version for Fedora 10

* Mon Dec  1 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-2
- updated sbcl version for Fedora 8 to 1.0.22 (same for x86_64 in fc9)

* Wed Nov 26 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-2
- updated lisp versions on fc9, rebuild

* Tue Nov 25 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- added support for fc10, removed build support for fedora <  7

* Fri Jul  4 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.02.25-1
- updated sbcl/cmucl versions to 19e and 1.0.17-3 for fc8/9, rebuild
- updated to last clm3 snapshot

* Tue May 13 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- added lisp versions for f9
- disabled cmucl build on fc9, hangs

* Fri Feb  1 2008 Arnaud Gomes-do-Vale <Arnaud.Gomes@ircam.fr>
- built on CentOS 5

* Fri Jan 18 2008 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2008.01.09-1
- updated to 2008.01.09 snapshot, rebuild for sbcl 1.0.13 on f7/8

* Wed Nov 14 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.10.10-1
- updated to 2007.10.10 snapshot
- added lisp versions for f8 build
- added --build-id to the linker call so that debuginfo is properly
  generated (see: http://fedoraproject.org/wiki/Releases/FeatureBuildId)

* Tue Sep  4 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.09.02-1
- updated to 2007.09.02 snapshot, build for sbcl 1.0.9-1

* Mon Jun 11 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.06.07-1
- updated to 2007.06.07 snapshot, build for sbcl 1.0.6
- added tcsh requirement

* Tue May  8 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.05.03-1
- updated to 2007.05.03 snapshot
- added fc7 support

* Mon Apr 23 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.04.20-1
- updated to 2007.04.20 snapshot, rebuild on fc5/fc6

* Mon Apr  2 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.03.22-1
- updated to 2007.03.22 snapshot, rebuild on new sbcl 1.0.4-1.1
  (with threading enabled)
- use release aware Requires: to make sure right sbcl is used

* Wed Mar 28 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.03.05-2
- rebuild for sbcl 1.0.4

* Fri Mar 16 2007 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2007.03.05-1
- updated to 2007.05.03 snapshot

* Thu Dec 21 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.12.12-1
- updated to 2006.12.12, rebuild on fc6/sbcl=1.0

* Wed Nov 29 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.11.29-1
- updated to latest snapshot
- name of configuration include file changed to mus-config.h

* Thu Nov 24 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.09.07-2
- rebuild on fc5 for sbcl 0.9.18

* Thu Sep  7 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.09.07
- update to latest snapshot

* Sat Jun 24 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.06.20
- update to latest snapshot

* Thu Feb 16 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.02.16
- includes fix for moog filter under sbcl 0.9.7 (some loop macro fix)

* Tue Jan 31 2006 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2006.01.31
- includes fix for grani under sbcl 0.9.7

* Tue Dec 13 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- overhaul to work with the new common music build system

* Thu Dec  8 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.12.05
- updated to 2005.12.05 snapshot

* Sun Dec  4 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.12.03
- updated to 2005.12.03 snapshot, start using asdf for building and 
  startup (but make it selectable at build time)

* Sat Nov 12 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.11.09
- updated to 2005.11.09 snapshot

* Fri Nov  4 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.11.04
- updated to clm 3 of 2005.11.04 (with cmucl 2005.11, sbcl 0.9.6)
- add sndlib.h to files copied to the source directory

* Tue Aug 16 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.08.14
- updated to clm 3 of 2005.08.14
- require cmucl 2005.08

* Mon Aug 15 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- sbcl dependenciy up to 0.9.3

* Tue Jul  5 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- updated sbcl and cmucl dependencies for fc4

* Thu May 12 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.05.11
- updated to 2005.05.11 snapshot, fix freeverb problem

* Thu May  5 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.05.03
- updated to 2005.05.03 snapshot

* Wed Feb 16 2005 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2005.02.16
- updated to 2005.02.16 snapshot

* Fri Dec 24 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.12.20
- updated to 2004.12.20
- spec file cleanup
- turn off style warnings in sbcl (thanks to Rick Taube)

* Fri Nov  5 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- split packages, created old clm2 packages for backwards compatibility,
  plain clm packages are now clm3 (this spec file)
- removed support for clisp, it is no longer supported in clm3
- added support for sbcl in clm3, using the asdf utility:
  http://www.cliki.net/asdf

* Wed Oct 20 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu>
- added patch to check for configure in the system directory in all.lisp
- todo: add default player?

* Sun Oct 17 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.08.30-1
- updated to end of life clm2 release
- add patch to include config.h from the bin-directory
- add patch1 to make-integer-array so that it uses signed integers under cmucl
- add feature to turn on signed arrays under cmucl (seems to work fine, at
  least with the 2004.10 cmucl snapshot)

* Wed May 19 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.05.19-1
- updated to 2004.05.19
- added /usr/bin/sndplay require to clm-clisp

* Tue May 11 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.05.06-1
- updated to 2004.05.06, build with cmucl 2004.05

* Sat May  8 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- added proper buildrequires

* Fri Mar 12 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.03.10-1
- updated to 2004.03.10

* Mon Feb  9 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.02.09-1
- updated to 2004.02.09

* Tue Feb  4 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 
- clisp: suppress redefinition warnings during load

* Mon Feb  2 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2004.02.02-1
- updated to 2004.02.02

* Wed Jan 28 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.28-1
- updated to 2004.01.28

* Wed Jan 21 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.15-1
- too many changes to document
- for now do not activate jack support for clisp
- cmucl: use the internal sndplay command with jack support

* Sat Jan 17 2004 Fernando Lopez-Lezcano <nando@ccrma.stanford.edu> 2003.12.15-1
- Initial build.

